class YAML_CONFIG_KEY():
    OBJECT_HOST = 'OBJECT_HOST'  # 项目host ，<dictionary>
    INTERFACE_URL = 'url'  # 用例接口路径<string>
    INTERFACE_REQUEST_DATA = 'req_data'  # 接请求参数 <dictionary>
    INTERFACE_ASSERT_DATA = 'ast_data'  # 断言参数<dictionary>
    INTERFACE_CASE_DES = 'des'  # 接口说明
    DING_TALK_URL = 'DING_TALK_URL'  # 钉钉URL<string>
    INTERFACE_JSON_PATH = 'json_expr'  # JSON_PATH表达式<string>
    INTERFACE_ASSERT_DATA_SETUP = 'setup'  # 用例前置操作
    INTERFACE_REQUEST_DATA_TEARDOWN = 'tearDown'  # 用例后置操作
    INTERFACE_REQUEST_HEADERS = 'headers'  # 用例请求头
    INTERFACE_CACHE_UPDATE_DATA = 'CACHE_UPDATE_DATA'
    INTERFACE_CACHE_METHOD = 'method'  # 用例请求方法<string>
    INTERFACE_CASE_SLEEP = 'sleep'  # 用例睡眠时长，单位秒<int>
    INTERFACE_CASE_HOST = 'host' # 用例绑定用户
    INTERFACE_DATA_BEFORE = 'before'  # 请求前执行函数
    INTERFACE_DATA_AFTER = 'after'  # 请求后执行函数



import jsonpath

class FindData():


    def find_expr_value(self, json_expr,find_obj):
        '''
        查找yaml文件数据
        :param json_expr:
        :return:
        '''
        new_json_expr = self.get_json_expr_find_expr(json_expr)

        # print('-------------')
        # print(new_json_expr)
        # print('-------------')
        # print(find_obj)


        index = self.get_json_expr_index(json_expr)
        json_value = jsonpath.jsonpath(find_obj, new_json_expr)
        if json_value == False: return json_expr
        if index != -1 and json_value:
            return json_value[index]
        return json_value


    def get_json_expr_index(self, json_expr):
        '''
        截取json_expr 下标
        :param json_expr:
        :return:
        '''
        if json_expr.find('/') == -1: return -1
        return int(json_expr.split('/')[-1])


    def get_json_expr_find_expr(self, json_expr):
        '''
        分割查找表达式
        :param json_expr:
        :return:
        '''
        if json_expr.find('/') == -1: return json_expr

        expr = json_expr.split('/')[0]

        if expr.find('-') == -1:
            expr_list = expr.split('.')
            expr_list.insert(1,'[*]')
            new_expr = '.'.join(expr_list)
            return new_expr

        return expr

if __name__ == '__main__':
    FindData().get_json_expr_find_expr('$.adminTenantCreate_data.adminTenantCreate_data.req_data.tenantName/0',True)


import os


class FileManage():

    def __init__(self):
        self._default_path = '.'

    def _get_obj_path(self):
        '''
        获取项目跟路径
        :return:
        '''
        yaml_file_path_list = []
        for dirpath, dirnames, filenames in os.walk(self._default_path):
            temp_obj = [yaml_file_name for yaml_file_name in filenames if yaml_file_name.endswith(".yaml")]
            if temp_obj:
                path = [os.path.join(dirpath, yaml_file_path) for yaml_file_path in temp_obj]
                yaml_file_path_list.extend(path)

        return yaml_file_path_list

    def get_obj_yaml_file_path(self):

        if self._get_obj_path():

            return self._get_obj_path()
        else:
            self._default_path = '..'
            return self._get_obj_path()






from ytApiTest.singleton import Singleton
from ytApiTest.fileManage import FileManage
from ytApiTest.findData import FindData
from ytApiTest.yamlKey import YAML_CONFIG_KEY

import yaml,jsonpath,copy


class YamlData(Singleton):
    _yaml_data_list = []
    def __init__(self):

        self._file = FileManage()
        self.find_data = FindData()

    def collect_skip_obj_yaml_data(self):

        '''
        查找项目所有yaml文件数据
        :return:
        '''

        if len(self._yaml_data_list):return self._yaml_data_list

        for path in self._file.get_obj_yaml_file_path():

            temp_dic = {}

            with open(path,encoding='UTF-8') as file:

                for yaml_dic in yaml.load_all(file,Loader=yaml.FullLoader):

                    temp_dic.update(yaml_dic)

            self._yaml_data_list.append(temp_dic)
        print('-------加载yaml文件数据--------')
        return self._yaml_data_list

    def get_interface_all_case_data(self,interface_name):
        '''
        获取接口所有用例数据
        :param interface_name:
        :return:
        '''
        interface_json_expr = '$.{}/0'.format(interface_name)
        find_value = self.find_data.find_expr_value(json_expr=interface_json_expr,find_obj=self.collect_skip_obj_yaml_data())
        return copy.deepcopy(find_value)

    def get_singleton_interface_case_data(self,interface_name,case_name):
        '''
        获取单个用例数据
        :param interface_name:
        :param case_name:
        :return:
        '''
        return self.get_interface_all_case_data(interface_name=interface_name).get(case_name)

    def get_case_interface_name(self,case_name):
        '''
        获取接口名
        :param case_name: 用例名
        :return:
        '''
        temp_value = copy.deepcopy(self.collect_skip_obj_yaml_data())
        for value in temp_value:
            if value.__contains__(YAML_CONFIG_KEY.OBJECT_HOST):
                value.pop(YAML_CONFIG_KEY.OBJECT_HOST)
            interface = [key for key ,v in value.items() if v != None and v.__contains__(case_name)]
            if len(interface):
                 return interface[0]

    def update_case_data(self,interface,case_name,req_data,update_key):
        '''请求数据
         更新用例
        :param interface:
        :param case_name:
        :param req_data:
        :return:
        '''

        if update_key == YAML_CONFIG_KEY.INTERFACE_REQUEST_DATA:
            
            expr = '$.{}.{}.{}'.format(interface,case_name,YAML_CONFIG_KEY.INTERFACE_REQUEST_DATA)

            for index,value in enumerate(self.collect_skip_obj_yaml_data()):

                find_value = jsonpath.jsonpath(value,expr)
                if find_value and find_value != None:
                    find_value = find_value[0]
                if isinstance(find_value,dict):
                    
                    find_value.update(req_data)
                
                elif find_value != False:
                    
                    self.collect_skip_obj_yaml_data()[index][interface][case_name][YAML_CONFIG_KEY.INTERFACE_REQUEST_DATA] = req_data
         
        elif case_name == YAML_CONFIG_KEY.INTERFACE_JSON_PATH:
            expr = '$.{}.{}.{}/0'.format(interface, case_name, YAML_CONFIG_KEY.INTERFACE_JSON_PATH)
            for index, value in enumerate(self.collect_skip_obj_yaml_data()):
                find_value = self.find_data.find_expr_value(find_obj=value, json_expr=expr)
                if find_value != False:
                    find_value.format(**req_data)

if __name__ == '__main__':
    pass


import threading
class Singleton:
    _instance_lock = threading.Lock()

    def __new__(cls, *args, **kwargs):
        if not hasattr(Singleton, "_instance"):
            with Singleton._instance_lock:
                if not hasattr(Singleton, "_instance"):
                    Singleton._instance = super(Singleton, cls).__new__(cls, *args, **kwargs)
        return Singleton._instance
